package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ch.ethz.ssh2.SCPClient;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;

/**
 * An extention of JDialog that imports a new command into OpView
 * from a log file.
 * @author Kathe Todd-Brown
 *
 */
@SuppressWarnings("serial")
public class ImportOpDialog extends JDialog {
	
	MainFrame frame;
	JButton okbutton, cancelbutton;
	JComboBox nonincludedRoots;
	JPanel buttonPane, comboPane, allPane;
	JCheckBox allBox;
	
	Vector <String> finalLogRoots;
	
	/**
	 * pulls an array of strings identifing the input files
	 * @param cline a command line string
	 * @return an array of strings identifying the input files
	 */
	String [] getInput(String cline){

		//a hash set that allows us to dynamically dump the input files
		//...we find, it also has the advantage of not double listing
		//...the input files
		HashSet<String> dynamicAns = new HashSet<String>();
		//split the commandline up based on empty spaces outside of quotes
		//...and strip the quotes
		String [] commandArray = GeneralUtil.stripAndSplit(cline);
		
		//go through the command
		for(int i = 0; i < commandArray.length; i ++){
			if(commandArray[i].contains(".") 
					&& (!commandArray[i].matches("\\d*.?\\d*"))
					&& (!commandArray[i].endsWith(".exe"))
					&& (!commandArray[i].endsWith(".exe\""))
					&& (!commandArray[i].endsWith("plink"))
					&& (!commandArray[i].endsWith("plink\""))){
				if(frame.data.isRemote(false) == true)
					dynamicAns.add(commandArray[i]);
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
							+ commandArray[i]);
			}
			
			if(commandArray[i].equals("--bfile")){
				String tempRoot = "plink";
				if( (i+1) < commandArray.length)
					tempRoot = commandArray[i+1];
			
				if(frame.data.isRemote(false) == true)
					dynamicAns.add(tempRoot + ".bed");
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
							+tempRoot + ".bed");
				
				if(frame.data.isRemote(false) == true)
					dynamicAns.add(tempRoot + ".fam");
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
						+tempRoot + ".fam");
				
				if(frame.data.isRemote(false) == true)
					dynamicAns.add(tempRoot + ".bim");
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
							+tempRoot + ".bim");
			}
			
			if(commandArray[i].equals("--file")){
				String tempRoot = "plink";
				if( (i+1) < commandArray.length)
					tempRoot = commandArray[i+1];
				
				if(frame.data.isRemote(false) == true)
					dynamicAns.add(tempRoot + ".map");
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
							+tempRoot + ".map");

				if(frame.data.isRemote(false) == true)
					dynamicAns.add(tempRoot + ".ped");
				else
					dynamicAns.add(frame.data.getRoot() + File.separator 
							+ tempRoot + ".ped");
			}
		}
			
		//make the answer
		int numInfiles = dynamicAns.size();
		String [] ans = new String [numInfiles];
		
		//copy the answer from the hash set
		int index = 0;
		for(String s: dynamicAns){
			if(frame.data.isRemote(false)){
				//assume that it's just the file name instead of path
				if(!s.contains("/")){
					s = frame.data.getRemoteFile() + "/" + s;
				}
			}
			ans[index] = s;
			index ++;
		}
		
		//return the answer
		return ans;
	}
	
	String getCommandline(String outputRoot){
		
		File logFile = new File(frame.data.getRoot(), outputRoot+".log");
		String cline = GeneralUtil.quoteFilename(frame.data.getPlinkPath());
		try {
			BufferedReader input = new BufferedReader(new FileReader(logFile));
			String line = input.readLine();
			while(line !=null){
				//System.out.println(line);
				if(line.contains("Options in effect")){
					line = input.readLine();
					//System.out.println(line);
					while(line != null
							&& !line.trim().equals("")){
						cline += " " + line.trim();
						//System.out.println(line);
						line = input.readLine();
					}
					input.close();
					return cline;
				}
				line = input.readLine();
			}
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException e2){
			e2.printStackTrace();
		}
		
		return cline;
	}
	
	@SuppressWarnings("static-access")
	void process(){
		/*
		 * Get the roots that we are importing
		 */
		Vector <String> selectedRoots = new Vector<String>();
		
		//are we importing everything or just one?
		if(allBox.isSelected())
			selectedRoots = finalLogRoots;
		else
			selectedRoots.add((String)nonincludedRoots.getSelectedItem());
		
		/*
		 * Process each root
		 */
		for(String root: selectedRoots){
			//if it's remote we assume that we had pulled the latest log
			//...file in our previous update cycle
		
			//pull the info from the associated log file
			String cline = getCommandline(root);
			
			String [] infiles = getInput(cline);
			String [] outfiles = new String[1];
			if(frame.data.isRemote(true) == true){
				outfiles[0] = frame.data.getRemoteFile()+ "/"
					+ root+".log";
			}
			else{
				outfiles[0] = frame.data.getRoot().getAbsolutePath()
					+ File.separator + root+".log";
			}
			
			String [] empty1 = new String [infiles.length];
			for(int i = 0; i < infiles.length; i++){
				empty1[i] = "";
			}
			
			String [] empty = {""};
			frame.data.addOperation(cline, root, "", infiles, empty1,
					outfiles, empty);
			
			//Assume that all imported operation are done and create
			//...a .gplink file if it's not already there.
			try {
				File gplinkfile = new  File(frame.data.getRoot(), root + frame.data.OP_STATUS_EXT);
				if(gplinkfile.createNewFile()){
					BufferedWriter out = new BufferedWriter(new FileWriter(gplinkfile));
				    out.write("0");
				    out.close();
				   
				    if( frame.data.isRemote(true)){
						SCPClient scopy = new SCPClient(frame.data.getConn());
						try {
							scopy.put((frame.data.getRoot().getAbsolutePath() 
									+ File.separator + root + frame.data.OP_STATUS_EXT),
									GeneralUtil.makeSpacesHard(frame.data.getRemoteFile()), frame.data.SCP_MODE);
						} catch (IOException e) {
							new ErrorDialog(frame, "Error trying to write the gplink file remotely.\n" +
									"process:ImportOpDialog.java");
						}
					}
				    
				}
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to create the .gplink file " +
						"for imported operation.\n process:ImportOpDialog()");
			}
		}
		
		//only pop up a edit dialog if we are importing
		//...a specified operation
		if(allBox.isSelected() == false){
			new EditOpDialog(selectedRoots.firstElement(), frame);
		}
		frame.updateViews();
	}
	
	void createRoots(){
		comboPane = new JPanel();
		comboPane.setLayout(new BoxLayout(comboPane, BoxLayout.PAGE_AXIS));
		comboPane.add(new JLabel("Select analysis to import:"));
		
		Vector <String> allLogRoots = new Vector <String>();
		
		Vector <String> knownOps = frame.data.getAllOpName();

		Vector <String> allFiles = frame.data.getFileList(); 
		Vector <String> allLogs = new Vector<String>();
		
		for(String file: allFiles){
			if(file.endsWith(".log"))
				allLogs.add(file);
		}
		
		finalLogRoots = new Vector <String>();
	
		for(String file: allLogs){
			String logRoot = file.substring(0, file.length()-4);
			logRoot = GeneralUtil.fileName(logRoot);
			allLogRoots.add(logRoot);
			if(!knownOps.contains(logRoot))
				finalLogRoots.add(logRoot);	
		}
		
		Collections.sort(finalLogRoots);
		nonincludedRoots = new JComboBox(finalLogRoots);	
		comboPane.add(nonincludedRoots);
	}
	
	void createButtons(){
		okbutton = new JButton("OK");
		cancelbutton = new JButton("Cancel");
		buttonPane = new JPanel();
		cancelbutton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				dispose();}}
		);
		
		okbutton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				process();
				dispose();}}
		);
		
		buttonPane.add(okbutton);
		buttonPane.add(cancelbutton);
	}
	
	private void createAllBox(){
		allBox = new JCheckBox("Import all operations.");
		allPane = new JPanel();
		allPane.add(allBox);
	}
	
	public ImportOpDialog(MainFrame mf){
		super(mf, "Import Analysis");
		frame = mf;
		
		createRoots();
		createAllBox();
		createButtons();
		
		getRootPane().setLayout(new BoxLayout(getRootPane(), BoxLayout.PAGE_AXIS));
		getRootPane().add(comboPane);
		getRootPane().add(allPane);
		getRootPane().add(buttonPane);
		
		pack();
		setVisible(true);
	}
}
